Index: optional.hpp
===================================================================
RCS file: /cvsroot/boost/boost/boost/optional/optional.hpp,v
retrieving revision 1.12
diff -u -r1.12 optional.hpp
--- optional.hpp	26 Jun 2006 18:01:36 -0000	1.12
+++ optional.hpp	22 May 2007 19:08:04 -0000
@@ -282,10 +282,10 @@
 
     // Destroys the current value, if any, leaving this UNINITIALIZED
     // No-throw (assuming T::~T() doesn't)
-    void reset() { destroy(); }
+    void reset() { destroy(); update_debug(); }
 
     // Replaces the current value -if any- with 'val'
-    void reset ( argument_type val ) { assign(val); }
+    void reset ( argument_type val ) { assign(val); update_debug(); }
 
     // Returns a pointer to the value if this is initialized, otherwise,
     // returns NULL.
@@ -431,6 +431,14 @@
 
     bool m_initialized ;
     storage_type m_storage ;
+
+  protected:
+#ifndef NDEBUG
+    T* m_value;
+    void update_debug() { m_value = is_initialized() ? &(get_impl()) : (T*)0; }
+#else
+    void update_debug() {}
+#endif
 } ;
 
 } // namespace optional_detail
@@ -455,19 +463,19 @@
 
     // Creates an optional<T> uninitialized.
     // No-throw
-    optional() : base() {}
+    optional() : base() { update_debug(); }
 
     // Creates an optional<T> uninitialized.
     // No-throw
-    optional( none_t const& none_ ) : base(none_) {}
+    optional( none_t const& none_ ) : base(none_) { update_debug(); }
 
     // Creates an optional<T> initialized with 'val'.
     // Can throw if T::T(T const&) does
-    optional ( argument_type val ) : base(val) {}
+    optional ( argument_type val ) : base(val) { update_debug(); }
 
     // Creates an optional<T> initialized with 'val' IFF cond is true, otherwise creates an uninitialized optional.
     // Can throw if T::T(T const&) does
-    optional ( bool cond, argument_type val ) : base(cond,val) {}
+    optional ( bool cond, argument_type val ) : base(cond,val) { update_debug(); }
 
 #ifndef BOOST_OPTIONAL_NO_CONVERTING_COPY_CTOR
     // NOTE: MSVC needs templated versions first
@@ -482,6 +490,7 @@
     {
       if ( rhs.is_initialized() )
         this->construct(rhs.get());
+      update_debug(); 
     }
 #endif
 
@@ -496,12 +505,12 @@
     // Depending on the above some T ctor is called.
     // Can throw is the resolved T ctor throws.
     template<class Expr>
-    explicit optional ( Expr const& expr ) : base(expr,&expr) {}
+    explicit optional ( Expr const& expr ) : base(expr,&expr) { update_debug(); }
 #endif
 
     // Creates a deep copy of another optional<T>
     // Can throw if T::T(T const&) does
-    optional ( optional const& rhs ) : base(rhs) {}
+    optional ( optional const& rhs ) : base(rhs) { update_debug(); }
 
    // No-throw (assuming T::~T() doesn't)
     ~optional() {}
@@ -513,6 +522,7 @@
     optional& operator= ( Expr expr )
       {
         this->assign_expr(expr,&expr);
+        update_debug(); 
         return *this ;
       }
 #endif
@@ -526,6 +536,7 @@
     optional& operator= ( optional<U> const& rhs )
       {
         this->assign(rhs);
+        update_debug(); 
         return *this ;
       }
 #endif
@@ -536,6 +547,7 @@
     optional& operator= ( optional const& rhs )
       {
         this->assign( rhs ) ;
+        update_debug(); 
         return *this ;
       }
 
@@ -544,6 +556,7 @@
     optional& operator= ( argument_type val )
       {
         this->assign( val ) ;
+        update_debug(); 
         return *this ;
       }
 
@@ -553,6 +566,7 @@
     optional& operator= ( none_t const& none_ )
       {
         this->assign( none_ ) ;
+        update_debug(); 
         return *this ;
       }
 
